@@ -1,5 +1,20 @@
 //! `salak_derive` provides a derive macro [`FromEnvironment`] for [salak](https://crates.io/crates/salak).
-
+#![cfg_attr(docsrs, feature(doc_cfg))]
+#![warn(
+    anonymous_parameters,
+    missing_copy_implementations,
+    missing_debug_implementations,
+    missing_docs,
+    nonstandard_style,
+    rust_2018_idioms,
+    single_use_lifetimes,
+    trivial_casts,
+    trivial_numeric_casts,
+    unreachable_pub,
+    unused_extern_crates,
+    unused_qualifications,
+    variant_size_differences
+)]
 use proc_macro::TokenStream;
 use quote::quote;
 use syn::*;
@@ -49,6 +64,17 @@ fn parse_attribute_prefix(attrs: &[Attribute]) -> Option<String> {
     None
 }
 
+fn disable_attribute_prefix_enum(attrs: &[Attribute]) {
+    for attr in attrs {
+        if let Ok(Meta::List(list)) = attr.parse_meta() {
+            if !is_salak(&list) {
+                continue;
+            }
+            panic!("Salak attribute is not supporting enum");
+        }
+    }
+}
+
 fn is_salak(list: &MetaList) -> bool {
     if let Some(v) = list.path.segments.iter().next() {
         return v.ident == "salak";
@@ -126,10 +152,10 @@ fn derive_field(field: Field) -> (quote::__private::TokenStream, quote::__privat
     let (def, def_desc) = parse_field_attribute(field.attrs, &mut rename);
     (
         quote! {
-            #name: env.require_def::<#ty, &str>(key, stringify!(#rename), #def)?
+            #name: env.require_def::<#ty>(stringify!(#rename), #def)?
         },
         quote! {
-            env.key_desc::<#ty, &str>(key, stringify!(#rename), #def_desc, keys);
+            env.add_key_desc::<#ty>(stringify!(#rename), #def_desc);
         },
     )
 }
@@ -157,22 +183,18 @@ fn derive_struct(name: &Ident, data: DataStruct) -> quote::__private::TokenStrea
     let (field, field_desc) = derive_fields(data.fields);
     quote! {
         impl FromEnvironment for #name {
-            fn from_env<'a>(
-                key: &mut Key<'a>,
+            fn from_env(
                 val: Option<Property<'_>>,
-                env: &'a impl Environment,
+                env: &mut SalakContext<'_>,
             ) -> Result<Self, PropertyError> {
                 Ok(Self {
                    #(#field),*
                 })
             }
+        }
 
-            fn key_desc<'a>(
-                key: &mut Key<'a>,
-                _: &mut KeyDesc,
-                keys: &mut Vec<KeyDesc>,
-                env: &'a impl DescribableEnvironment,
-            ) {
+        impl DescFromEnvironment for #name {
+            fn key_desc(env: &mut SalakDescContext<'_>) {
                 #(#field_desc)*
             }
         }
@@ -182,6 +204,7 @@ fn derive_struct(name: &Ident, data: DataStruct) -> quote::__private::TokenStrea
 fn derive_enum(type_name: &Ident, data: &DataEnum) -> quote::__private::TokenStream {
     let mut vs = vec![];
     for variant in &data.variants {
+        disable_attribute_prefix_enum(&variant.attrs);
         let lname = quote::format_ident!("{}", format!("{}", variant.ident).to_lowercase());
         let name = &variant.ident;
         let body = match variant.fields {
@@ -227,7 +250,10 @@ pub fn from_env_derive(input: TokenStream) -> TokenStream {
             },
             derive_struct(&name, d),
         ),
-        Data::Enum(d) => (quote! {}, derive_enum(&name, &d)),
+        Data::Enum(d) => {
+            disable_attribute_prefix_enum(&input.attrs);
+            (quote! {}, derive_enum(&name, &d))
+        }
         _ => panic!("union is not supported"),
     };
 
